var path = require('path');
var fs = require('fs');
var chalk = require('chalk');
var request = require('request');
var debug = require('debug')('wevue');
var plugin = {};

plugin.add = function(wxFolder, pkg, registry, cb) {
    var plugFile = path.join(wxFolder, 'plugin.json');
    debug('plugFile', plugFile);
    var packageJSON = {};
    if (fs.existsSync(plugFile)) {
        var packageContent = fs.readFileSync(plugFile, 'utf8');
        packageJSON = JSON.parse(packageContent);
    }

    if (pkg.indexOf('@') > 0) {
        var name = pkg.substr(0, pkg.indexOf('@'));
        var version = pkg.substr(pkg.indexOf('@') + 1);
        packageJSON[name] = version;
        content = JSON.stringify(packageJSON, null, 4);
        debug(content);
        fs.writeFileSync(plugFile, content);
        cb(null);
    } else {
        getLastVersion(pkg, registry, function(err, version) {
            debug('version', version);
            if (!err) {
                packageJSON[pkg] = version;
                content = JSON.stringify(packageJSON, null, 4);
                debug(content);
                fs.writeFileSync(plugFile, content);
                cb(null);
            } else {
                cb(err);
            }
        });
    }
}

plugin.getInstallCmd = function(wxFolder) {
    var plugFile = path.join(wxFolder, 'plugin.json');
    if (fs.existsSync(plugFile)) {
        var packageContent = fs.readFileSync(plugFile, 'utf8');
        var packageJSON = JSON.parse(packageContent);
        var plugins = [];
        for (var name in packageJSON) {
            if (packageJSON.hasOwnProperty(name)) {
                plugins.push(name+'@'+packageJSON[name]);
            }
        }
        debug(packageJSON);
        debug(plugins);
        if (plugins.length > 0) {
            return 'npm install ' + plugins.join(" ");
        }
    }
    return null;
}


function getLastVersion(plugin, registry, cb) {
    var url = (registry ? registry : `https://registry.npmjs.org`) + "/" + plugin;
    debug('request url:' + url);

    request(url, function(error, response, body) {
        if (!error && response.statusCode === 200) {
            var res = JSON.parse(body);
            var lastVersion = res["dist-tags"].latest;
            cb(null, lastVersion)
        } else {
            console.log(chalk.yellow("warn:get plugins veriosn fail from " + url));
            debug(error);
            cb(error);
        }
    })
}

module.exports = plugin;
